Attribute VB_Name = "mCamera"
Option Explicit

'Type
    Public Type tCamera
        'Current position
        X As Single
        Y As Single
        
        'Requested position
        DestX As Long
        DestY As Long
        
        'Movement angle
        Angle As Single
        
        'Screen size
        w As Long 'W means width
        h As Long 'H means height
        
        'Number of tiles that fit on screen
        TilesX As Long
        TilesY As Long
        
        'Front buffer
        FrontDC As Long
    End Type

'Variables
    Public Camera As tCamera

Private Sub Clip()
    With Camera
        'Check upper boundaries
        If (.X + .w) > (Map.w * TileSize) Then: .X = (Map.w * TileSize) - .w
        If (.Y + .h) > (Map.h * TileSize) Then: .Y = (Map.h * TileSize) - .h
        
        'Check lower boundaries
        If .X < 0 Then: .X = 0
        If .Y < 0 Then: .Y = 0
    End With
End Sub

Public Sub LookAt(iX As Long, iY As Long, iSmooth As Boolean)
    With Camera
        'Set requested position
        .DestX = iX
        .DestY = iY
        
        If iSmooth Then
            'Calculate new angle
            .Angle = GetAngle(.X, .Y, CSng(.DestX), CSng(.DestY))
        
        Else
            'Set current position
            .X = iX
            .Y = iY
            
            'Check position
            Clip
        End If
    End With
End Sub

Public Sub Reset(iFront As PictureBox)
    With Camera
        'Get viewport size
        .w = iFront.ScaleWidth
        .h = iFront.ScaleHeight
        
        'Calculate number of visible tiles
        .TilesX = (.w / TileSize)
        .TilesY = (.h / TileSize)
        
        'Get device context
        .FrontDC = iFront.hDC
    End With
End Sub

Public Sub UpdatePosition()
    Dim Dist As Long
    Static NextMove As Long
    
    With Camera
        'Timing
        If NextMove > GetTickCount Then: Exit Sub
        NextMove = GetTickCount + 20 'Update at 50 FPS
        
        'Get distance
        Dist = Distance(CLng(.X), CLng(.Y), .DestX, .DestY)
        If Dist = 0 Then: Exit Sub
        
        'Limit speed
        If Dist > 128 Then: Dist = 128
        
        'Move camera
        .X = .X + Cos(.Angle) * (Dist * 0.2)
        .Y = .Y + Sin(.Angle) * (Dist * 0.2)
        Clip
    End With
End Sub


